/*
  Joakim Wesslen
  2012-06-26
  Modified 2015-07-16: Ben Smeets
  Added hooks for encryption and changed methods APIs a bit
  Not tested for dimming devices

  A library to handle the TX433 Proove/Anslut device.
  Version 1.0
  
  Based on code from
  http://tech.jolowe.se/home-automation-rf-protocols/
*/
#include "tx433.h"
#include "TRXtypes.h"



#define ENCRYPTION
// comment out next line to include encryption using a cipher that can encrypt a block of 32 bits
#undef ENCRYPTION


#ifdef ENCRYPTION
// API for Encryption of a 32bit word using a 128bit key
#include "skip32.h"
//	void encrypt32 (unsigned long *data);
//	void decrypt32 (unsigned long *data);
#endif

#define RETRANSMIT  5

const int width = 250;

const int Tx433::pulse_high = width;
const int Tx433::pulse_one_low = width-50;
const int Tx433::pulse_zero_low = 5*width;
const int Tx433::pulse_sync_low = 10*width;
const int Tx433::pulse_pause_low =40*width;

/* 4 bit code for the receiving unit, depends on Proove or Nexa
 * if channel=0: It is Proove and dev is 0:00, 1=01, 2=10
 * if channel=3. It is Nexa and dev is 0:11, 1=10, 2=01
*/
static unsigned int DevControl(unsigned int channel, int dev)
{
  unsigned int mask;
  unsigned int cntr = (channel << 2);
	if ((channel==0)&&(dev < 3)){
          // Proove/Anslut:
            cntr |= dev;
	}
	else if ((channel==3)&&(dev < 3)){
          // Nexa
	    cntr |= (dev ^ channel);
	}
	else
	    cntr |=dev;
  return cntr;
}


/* Public */

Tx433::Tx433(){
};

Tx433::Tx433(int digitalpin, unsigned long transmittercode, unsigned int channelcode)
{
  baseInit(digitalpin, transmittercode, channelcode);
  keyset = false;
}


Tx433::Tx433(int digitalpin, unsigned long transmittercode, unsigned int channelcode, unsigned char encryptkey[16])
{
  baseInit(digitalpin, transmittercode, channelcode);
  memcpy(key, encryptkey, 16);
  keyset = true;
}

void Tx433::setKey(unsigned char encryptkey[16])
{
  memcpy(key, encryptkey, 16);
  keyset = true;
}  
  

void Tx433::Device_On(unsigned int dev)
{ 
  sendPackets(dev, true); 
}

void Tx433::Device_Off(unsigned int dev)
{
  sendPackets(dev, false); 
}

/* Private */

void Tx433::baseInit(int digitalpin, unsigned long transmittercode, unsigned int channelcode) {
  int i;
  TxCode = transmittercode;
  txpin = digitalpin;
  pinMode(txpin, OUTPUT);
  channelCode = channelcode;
}

// Encrypt
static void encrypt(BYTE key[], unsigned long *data)
{
#ifdef ENCRYPTION
	// inefficient to make a new encryption engine each time we encrypt but
	// in this we avoid adding the engine as member of the Tx433 class
	skip32 crypt(key, 16);
	crypt.encrypt32(data);
#endif
}


// Protocol layer,
// Manchester like code, each symbol is repeated in complement form
// Packet: sync+code+pause,
void Tx433::sendCode(unsigned long code, int len) {
  int i = 0;
  unsigned long mask = 1;
  mask <<= (len-1);
  sendSync();
  while (i < len) {
    if (mask & code) {
      sendOne();
      sendZero();
      Serial.print("1");
	}
	else {
      sendZero();
      sendOne();
      Serial.print("0");
    }
    mask >>= 1;
    i++;
  }
  sendPause();
  Serial.println();
}
/* Send the on/off data to the receiving device
/*         data=TxCode+group+on+ChCode/dim+dev
/*         Packets are send RETRANSMIT times in a row
*/
void Tx433::sendPackets(unsigned int dev, bool on) {
  unsigned long data;
  unsigned int group;
  dev = (dev < 3) ? dev : 3;
  group = (dev == 3) ? 1 : 0;
  data = TxCode;
  data = (data << 1) | (1^ group);
  data = (data << 1) | (1^ on);
  data = (data << 4) | (DevControl(channelCode, dev));
#ifdef ENCRYPTION
  // Encrypt the data
  if (keyset)
	  encrypt(Tx433::key, &data);
  else
	  Serial.println("error");
#endif
// send the package the specified number of times
  for (int i = 0; i < RETRANSMIT; i++) {
    sendCode(data, 32);   
  }
}

/**
 * Physical layer
 * below the routine that send the logical symbols
 */
void Tx433::sendZero() {
  digitalWrite(txpin, HIGH);
  delayMicroseconds(pulse_high);
  digitalWrite(txpin, LOW);
  delayMicroseconds(pulse_zero_low);
}

void Tx433::sendOne() {
  digitalWrite(txpin, HIGH);
  delayMicroseconds(pulse_high);
  digitalWrite(txpin, LOW);
  delayMicroseconds(pulse_one_low);
}

void Tx433::sendSync() {
  digitalWrite(txpin, HIGH);
  delayMicroseconds(pulse_high);
  digitalWrite(txpin, LOW);
  delayMicroseconds(pulse_sync_low);
}

void Tx433::sendPause() {
  digitalWrite(txpin, HIGH);
  delayMicroseconds(pulse_high);
  digitalWrite(txpin, LOW);
  delayMicroseconds(pulse_pause_low);
}


