#ifndef TX433_H
#define TX433_H
/*

  TX class for sending commands to Nexa or Proove power switch receivers
  using a simple 433Mhz transmitter



  Code is inspired by a solution of
  Joakim Wesslen
  2012-06-26


 */

#include "Arduino.h"

class Tx433
{
public:
	/*
	@digitalpin - the digitalpin to send data on to transmitter
	@transmittercode - the unique code of the transmitter (26 bits)
	@channelcode - the channel code (2 bits)
	 */
	Tx433();
	Tx433(int digitalpin, unsigned long transmittercode, unsigned int channelcode);
	Tx433(int digitalpin, unsigned long transmittercode, unsigned int channelcode, unsigned char encryptkey[16]);

	/*
	@unit - the device to turn on. 
		0,1,2 are the three separate devices.
		3 is the complete group.
	 */
	void Device_On(unsigned int unit);	 

	/*
	@unit - the device to turn off. 
		0,1,2 are the three separate devices.
		3 is the complete group.
	 */
	void Device_Off(unsigned int unit);

	void baseInit(int digitalpin, unsigned long transmittercode, unsigned int channelcode);
	void setKey(unsigned char encryptkey[16]);

private:
	int txpin;
	unsigned char key[16];
	bool keyset;

	unsigned long TxCode;
	unsigned int channelCode;

	const static int pulse_high;
	const static int pulse_one_low;
	const static int pulse_zero_low;
	const static int pulse_sync_low;
	const static int pulse_pause_low;

	// Protocol layer
	void sendCode(unsigned long data, int len);
	void sendPackets(unsigned int unit, bool on);

	// Physical layer interfaces
	void sendZero(void);
	void sendOne(void);
	void sendSync(void);
	void sendPause(void);

};
#endif	// TX433_H
